/*
  Copyright 1999 Brian H. Trammell (brian@altara.org)
  Copyright 1999 Ben Parrott (benji@altara.org)
  
  This software (and its source code) are freely usable, modifiable, 
  and redistributable under the terms of the GNU General Public License, 
  available at http://www.gnu.org/copyleft/gpl.html, and in the ./COPYING 
  file in the MARS distribution.
*/

package org.altara.mars;

import javax.swing.*;
import javax.swing.table.*;
import javax.swing.event.*;
import java.awt.*;
import java.awt.event.*;
import java.util.*;

/**
   This class creates the main UI interface, and sets up action and selection
   listeners to handle user input.
*/

// (c) 1999 Brian H. Trammell and Ben Parrott

// Revision 1.1: 12 Mar 1999 bht - added support for status bar, added
//                                 advanced table properties (color-coded
//                                 displays via TableRenderer)
// Revision 1.2.1: 21 Mar 1999 bht - added DisplayDownAlert() for new
//                                   alert semantics. changed background
//                                   color for timed out services.
// Revision 1.2.2: 25 Mar 1999 bht - cleaned up table renderer stuff
// Revision 1.2.5: 03 Apr 1999 bht - added support for ListSelectionListener,
//                                   changed color for ok services.
// Revision 1.3.0: 06 Apr 1999 bht - updated to reflect name change of
//                                   ButtonAction to MARSAction, and for new
//                                   MARSAction constants.

public class MARSMainUI extends JFrame
implements ListSelectionListener
{

/** compile-time debug flag. Should be false */
public final static boolean DEBUG = false;
/** width of the status columns in the main table */
public final static int STATWIDTH = 45;
/** width of the hostname column in the main table */
public final static int HOSTWIDTH = 180;

/* i like to scroll */
JScrollPane scrollPane;
/* host model for the main display */
MARSTableModel hostData;
/* the main display */
JTable hostTable;
/* static reference to the current UI window. Used for warnings and status
   bars and the like */
static MARSMainUI currentUI;
/* the toolbar */ 
JToolBar buttonBar;
/* the Stop button */
JButton stopButton;
/* the Start button */
JButton startButton;
/* the Config button */
JButton configButton;
/* the Quit button */
JButton quitButton;
/* the status bar. Whee! */
JLabel statusBar;
/* Tells us if we are in the run state or the stopped state */
public boolean running = true;

/**
 *  Create a new main window.
 *  The MARSMainUI will create the needed table based on the configuration,
 *  set up action listners for the buttons, and basically act as the user
 *  interface for MARS 
 */
public MARSMainUI()
{

  /* Sets the string at the top of the window (wish I knew how to get
   * rid of that damn coffee cup)
   */
  super("MARS "+MARS.VERSION);

  /* Create the tool bar */
  buttonBar = new JToolBar();

  /* Create and store the Stop button */
  MARSAction stopAction = new MARSAction(MARSAction.STOP);
  stopButton = buttonBar.add(stopAction);
  /* Create and store the Start button */
  MARSAction startAction = new MARSAction(MARSAction.START);
  startButton = buttonBar.add(startAction);
  /* Make the Start button "unlit" */
  startButton.setVisible(false);
  /* Create and store the Config button */
  MARSAction configAction = new MARSAction(MARSAction.CONFIGURE);
  configButton = buttonBar.add(configAction);
  /* Create and store the Quit button */
  MARSAction quitAction = new MARSAction(MARSAction.QUIT);
  quitButton = buttonBar.add(quitAction);

  /* Create a window listener to shutdown MARS when close box is clicked. */
  addWindowListener(new WindowAdapter() {
           public void windowClosing(WindowEvent ev) {
	       MARS.shutdown();
	   }});

  /* Get a new TableModel */
  hostData = new MARSTableModel();
                
  /* Create the hostTable from our Table Model */
  hostTable = new JTable(hostData);

  /* here comes the heavy table wizardry. */
  /* Make the table cell backgrounds change color based on their status.
     Red means down, yellow means timed out, green means OK, white means not
     available. */
  /* Start by creating a cell renderer that can change color based on value */
  DefaultTableCellRenderer statusColumnRenderer =
      new DefaultTableCellRenderer() {
	  public void setValue(Object value) {
	      /* Possibly pointless validation. Make sure the cell is a
	         string. If it's not a string, everything's just gone right
                 to hell. :) */
	      String cellValue = (value instanceof String) ? 
	                           (String)value : "";
	      if (cellValue.equals(Host.TO))        // timeouts are yellow
	          setBackground(Color.yellow);
	      else if (cellValue.equals(Host.FAIL)) // failures are red
	          setBackground(Color.red);
	      else if (cellValue.equals(Host.OK))   // i like a green board
	          setBackground(Color.green);
	      else 
	          setBackground(Color.white);       // all else is blank
	      setText(cellValue);                   // now pass the value
	  }
      };
  
  /* Now get each status column and set its cell renderer and width */
  for (int i=0; i < MARS.SVC_COUNT; i++) {
      hostTable.getColumn(MARS.SVC[i]).setCellRenderer(statusColumnRenderer);
      hostTable.getColumn(MARS.SVC[i]).setPreferredWidth(STATWIDTH);
  }
  /* set the background color and cell width of the host column */
  DefaultTableCellRenderer hostColumnRenderer =
      new DefaultTableCellRenderer();
  hostColumnRenderer.setBackground(Color.lightGray);
  hostTable.getColumn("Host").setCellRenderer(hostColumnRenderer);
  hostTable.getColumn("Host").setPreferredWidth(HOSTWIDTH);

  /* more wizardry:
     enable only cell selection, and set up the click listener for the table 
     to be this UI. */
  hostTable.setCellSelectionEnabled(true);
  hostTable.setColumnSelectionAllowed(false);
  hostTable.setRowSelectionAllowed(false);
  hostTable.getSelectionModel().addListSelectionListener(this);

  /* end of heavy wizardry */

  /* Add the table to the scroll pane */
  scrollPane = new JScrollPane(hostTable);

  /* Create the status bar with a welcome message */
  statusBar = new JLabel("Welcome to MARS "+MARS.VERSION);
  
  /* Set the layout manager */
  this.getContentPane().setLayout(new BorderLayout());

  /* Adds the three critical items to the box */
  this.getContentPane().add(buttonBar,BorderLayout.NORTH);
  this.getContentPane().add(scrollPane,BorderLayout.CENTER);
  this.getContentPane().add(statusBar,BorderLayout.SOUTH);

  this.setSize(450,250);

  currentUI = this;

}

/** 
 * Used so we know if things are running or not 
 */
public boolean returnRunningStatus()
{
  return running; 
}

/**
   Used so other classes can access the MARSMainUI.
   This is needed so other methods can be called with a MARSMainUI as the
   frame.
 */

public static MARSMainUI getCurrent()
{
 return currentUI;
}

/**
 * Handle presses of the stop button
 */
public void stopPressed()
{
   /* Stop MARS */
   MARS.stopThreads();
   /* Clear the probe thread list. */
   ProbeThread.clearAll();

   /* Change what the pause button looks like */
   stopButton.setVisible(false);
   /* Change what the resume button looks like */
   startButton.setVisible(true);
   /* Set the running boolean to false */
   running = false;
}

/**
 * Handle presses of the start button
 */
public void startPressed()
{
   /* Reinitialize the ProbeThreads */
   MARS.initProbes();
   /* Restart all the threads */
   MARS.startThreads();

   /* Change what the pause button looks like */
   stopButton.setVisible(true);
   /* Change what the resume button looks like */
   startButton.setVisible(false);
   /* Set the running boolean to true */
   running = true;
}


/**
 * Handle presses of the config button
 */
public void configPressed()
{
    if (MARSConfigUI.getCurrent() == null)
	new MARSConfigUI().setVisible(true);
}

/**
 * Handle presses of the Quit button
 */
public void quitPressed()
{
    MARS.shutdown();
}


/**
 * Displays a status change alert for a specified host and service list.
 * @param host Screen name of the host
 * @param smtp True if changed, false if not changed
 * @param pop  True if changed, false if not changed                            
 * @param http True if changed, false if not changed
 * @param ftp  True if changed, false if not changed
 * @param frame The frame that will display the dialog box
 *
 *
 */

public static void DisplayChanges(String host, boolean smtp, boolean pop,
                                  boolean http, boolean ftp, JFrame frame)
 {
  /* The default starting message */
  String outputMessage = "The following services have changed status on ";
  /* The top of the message box*/
  String topMessage = "WARNING: Change on host ";

  /* Adds the host name to the output message */
  outputMessage = outputMessage + host + ":\n";
  /* Adds the host name to the window label*/
  topMessage = topMessage + host;

  /* Check SMTP; if it has changed, add the string */
  if (smtp)
     outputMessage = outputMessage + " " + "SMTP";
  /* Check POP; if it has changed, add the string */
  if (pop)
     outputMessage = outputMessage + " " + "POP";
  /* Check HTTP; if it has changed, add the string */
  if (http)
     outputMessage = outputMessage + " " + "HTTP";
  /* Check FTP; if it has changed, add the string */
  if (ftp)
     outputMessage = outputMessage + " " + "FTP";
 

  /* Displays the Message Box */
  JOptionPane.showMessageDialog(frame, outputMessage, topMessage,
                                JOptionPane.WARNING_MESSAGE);

 }

/**
   Displays an alert notifying the user that a SPOTS parameter went out of
   bounds. This exists to support older alert semantics, and is on the way
   out.

   @param parameter String specifying which parameter is out of bounds.
   @param host Name of host
   @param value Value of the offending parameter (String form)
   @param frame Frame to display this dialog over.
*/

public static void DisplayBoundsAlert (String parameter, String host,
				       String value, JFrame frame) {
    /* The warning message */
    String outputMessage = parameter+" of "+value+" is out of bounds on "+
	host;
    /* The top of the message box*/
    String topMessage = "WARNING: "+parameter+" on "+host;

    JOptionPane.showMessageDialog(frame,outputMessage,topMessage,
				  JOptionPane.WARNING_MESSAGE);
}

/**
   Displays an alert notifying the user that a service on a specified host
   is down. 

   @param service String specifying which service is down.
   @param host Name of host
   @param frame Frame to display this dialog over.
*/

public static void DisplayDownAlert (String service, String host,
				     JFrame frame) {
    /* The warning message */
    String outputMessage = "The "+service+" service is down on "+host+".";
    /* The top of the message box */
    String topMessage = "WARNING: "+service+" on "+host;

    JOptionPane.showMessageDialog(frame,outputMessage,topMessage,
				  JOptionPane.WARNING_MESSAGE);
}

/**
   Changes the message displayed in the status bar
*/

public void setStatus(String newStatus) {
    statusBar.setText(newStatus);
}

/**
   Handles a ListSelectionEvent for the main table. This code dispatches
   mouse clicks in the main table, and will one day be hooked up to detail
   views and configuration options.

   @param lse a ListSelectionEvent generated by a user click on the table
*/
public void valueChanged(ListSelectionEvent lse) {
    if ((!lse.getValueIsAdjusting()) ||
	hostTable.getSelectionModel().isSelectionEmpty())
	return;                    // makes each click register only once
                                   // does not register empty table events
    int row = hostTable.getSelectedRow(); // make row and column data available
    int col = hostTable.getSelectedColumn();     // within this method

    if (DEBUG) 
	System.out.println("MARSMainUI.vc(): click at ("+row+","+col+")");

    // unselect the current table selection
    // this is an event monitor, after all, not a table selector
    hostTable.getSelectionModel().clearSelection();
    
}

} // end class MARSMainUI












